From 3c0bfa0b877c1f5a015d6f849c9a5516885de7d1 Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@free.fr>
Date: Mon, 22 Feb 2010 17:20:27 +0100
Subject: [PATCH 2/5] input: Fill in the input_item es.

Instead of using es_format_t we use a custom structure to limit the memory footprint.
This allow libvlc_media_get_es() to properly work after libvlc_media_parse().
---
 include/vlc_input_item.h |   38 ++++++++++++++++++++++++++++++++++++++
 src/input/es_out.c       |   27 ++++++++++++++++++++++++++-
 2 files changed, 64 insertions(+), 1 deletions(-)

diff --git a/include/vlc_input_item.h b/include/vlc_input_item.h
index d4858c9..1e10acd 100644
--- a/include/vlc_input_item.h
+++ b/include/vlc_input_item.h
@@ -51,6 +51,44 @@ struct info_category_t
     struct info_t **pp_infos;     /**< Pointer to an array of infos */
 };
 
+/* Stripped down version of es_format_t because es_format_t can become very
+ * large in term of memory. This is mostly libvlc_media_es_t */
+typedef struct input_item_es_t {
+    vlc_fourcc_t i_codec;
+    int          i_id;
+    int          i_cat; /* @see es_format_category_e */
+
+    /* Codec specific */
+    int         i_profile;
+    int         i_level;
+
+    /* Audio specific */
+    unsigned    i_channels;
+    unsigned    i_rate;
+
+    /* Video specific */
+    unsigned    i_height;
+    unsigned    i_width;
+} input_item_es_t;
+
+static inline input_item_es_t *input_item_es_new_from_format(const es_format_t *fmt)
+{
+    input_item_es_t *ret = (input_item_es_t *)malloc(sizeof(input_item_es_t));
+    ret->i_codec = fmt->i_codec;
+    ret->i_id = fmt->i_id;
+    ret->i_cat = fmt->i_cat;
+
+    ret->i_profile = fmt->i_profile;
+    ret->i_level = fmt->i_level;
+
+    ret->i_channels = fmt->audio.i_channels;
+    ret->i_rate = fmt->audio.i_rate;
+
+    ret->i_height = fmt->video.i_height;
+    ret->i_width = fmt->video.i_width;
+    return ret;
+}
+
 struct input_item_t
 {
     VLC_GC_MEMBERS
diff --git a/src/input/es_out.c b/src/input/es_out.c
index 3062d8e..c2fb21d 100644
--- a/src/input/es_out.c
+++ b/src/input/es_out.c
@@ -2223,7 +2223,7 @@ static int EsOutControlLocked( es_out_t *out, int i_query, va_list args )
             }
             return VLC_SUCCESS;
         }
- 
+
         case ES_OUT_SET_ES_DEFAULT:
         {
             es_out_id_t *es = va_arg( args, es_out_id_t * );
@@ -2811,6 +2811,31 @@ static void EsOutUpdateInfo( es_out_t *out, es_out_id_t *es, const es_format_t *
     const es_format_t *p_fmt_es = &es->fmt;
     lldiv_t         div;
 
+    /* Update input item */
+    input_item_t *item = p_input->p->p_item;
+    vlc_mutex_lock( &item->lock );
+    bool found = false;
+
+    input_item_es_t *input_item_fmt = input_item_es_new_from_format(fmt);
+
+    for( int i = 0; i < item->i_es; i++ )
+    {
+        if (item->es[i]->i_id == es->i_id)
+        {
+            /* We've found the right ES, replace it */
+            free(item->es[i]);
+            item->es[i] = input_item_fmt;
+            found = true;
+            break;
+        }
+    }
+    if( !found )
+    {
+        /* ES not found, create a new one. */
+        TAB_APPEND(item->i_es, item->es, input_item_fmt);
+    }
+    vlc_mutex_unlock( &item->lock );
+
     /* Create category */
     char psz_cat[128];
     snprintf( psz_cat, sizeof(psz_cat),_("Stream %d"), es->i_meta_id );
-- 
1.7.0

